//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#include  "clsRegistry.h"

#include  <tchar.h>


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

cls_Registry::cls_Registry(HKEY new_hKey, LPCTSTR new_key, bool create_sw, bool no_close)
{
	hKey = NULL;
	if(create_sw)
		Create(new_hKey, new_key);
	else
		Open(new_hKey, new_key, no_close);
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::Open(HKEY new_hKey, LPCTSTR new_key, bool no_close)
{
	Close();
	if(!new_key || !*new_key)
	{
		hKey = new_hKey;
		NoClose = no_close;
		LastError = ERROR_SUCCESS;
		return true;
	}
	hKey = OpenRegKey(new_hKey, new_key);
	NoClose = false;
	LastError = GetLastError();
	return hKey != NULL;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

bool  cls_Registry::Create(HKEY new_hKey, LPCTSTR new_key)
{
	Close();
	hKey = CreateRegKey(new_hKey, new_key);
	NoClose = false;
	LastError = GetLastError();
	return hKey != NULL;
}


//////////////////////////////////////////////////////////////////////

void  cls_Registry::Close()
{
	if(hKey)
	{
		if(!NoClose)
			RegCloseKey(hKey);
		hKey = NULL;
	}
	NoClose = true;
	LastError = ERROR_SUCCESS;
}


//////////////////////////////////////////////////////////////////////

DWORD  cls_Registry::SubKeyCount()
{
	DWORD  count;
	LastError = (DWORD)RegQueryInfoKey(hKey, NULL, NULL, NULL, &count, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	if(LastError != ERROR_SUCCESS)
		return 0;
	return count;
}


//////////////////////////////////////////////////////////////////////

DWORD  cls_Registry::ValueCount()
{
	DWORD  count;
	LastError = (DWORD)RegQueryInfoKey(hKey, NULL, NULL, NULL, NULL, NULL, NULL, &count, NULL, NULL, NULL, NULL);
	if(LastError != ERROR_SUCCESS)
		return 0;
	return count;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::DeleteAllSubKeys()
{
	for(; ; )
	{
		LPTSTR  buf = GetRegSubKey(hKey, 0);
		if(!buf)
		{
			LastError = GetLastError();
			if(LastError == ERROR_NO_MORE_ITEMS)
			{
				LastError = ERROR_SUCCESS;
				return true;
			}
		}
		else
		{
			DeleteRegKeyTree(hKey, buf);
			LastError = GetLastError();
			free(buf);
		}
		if(LastError != ERROR_SUCCESS)
			return false;
	}
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::DeleteAllValues()
{
	for(; ; )
	{
		LPTSTR  buf = GetRegValueName(hKey, 0);
		if(!buf)
		{
			LastError = GetLastError();
			if(LastError == ERROR_NO_MORE_ITEMS)
			{
				LastError = ERROR_SUCCESS;
				return true;
			}
		}
		else
		{
			DeleteRegValue(hKey, buf);
			LastError = GetLastError();
			free(buf);
		}
		if(LastError != ERROR_SUCCESS)
			return false;
	}
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

LPBYTE  cls_Registry::GetBinary(LPCTSTR lpValueName, DWORD &length)
{
	RegValue  reg_value;
	if(GetValue(lpValueName, reg_value))
	{
		switch(reg_value.Type)
		{
			case REG_NONE:
			case REG_BINARY:
			case REG_LINK:
			case REG_RESOURCE_LIST:
			case REG_FULL_RESOURCE_DESCRIPTOR:
			case REG_RESOURCE_REQUIREMENTS_LIST:
				length = reg_value.Size;
				return reg_value.Data;
		}
	}
	free(reg_value.Data);
	if(LastError == ERROR_SUCCESS)
		LastError = ERROR_INVALID_DATA;
	return NULL;
}


//////////////////////////////////////////////////////////////////////

LPTSTR  cls_Registry::GetString(LPCTSTR lpValueName)
{
	RegValue  reg_value;
	if(GetValue(lpValueName, reg_value))
	{
		switch(reg_value.Type)
		{
			case REG_SZ:
			case REG_EXPAND_SZ:
			case REG_MULTI_SZ:
				return LPTSTR(reg_value.Data);
		}
	}
	free(reg_value.Data);
	if(LastError == ERROR_SUCCESS)
		LastError = ERROR_INVALID_DATA;
	return NULL;
}


//////////////////////////////////////////////////////////////////////

DWORD  cls_Registry::GetDWord(LPCTSTR lpValueName, DWORD default_value)
{
	RegValue  reg_value;
	if(GetValue(lpValueName, reg_value) && (reg_value.Type == REG_DWORD || reg_value.Type == REG_DWORD_BIG_ENDIAN))
	{
		default_value = *(DWORD *)reg_value.Data;
		if(reg_value.Type == REG_DWORD_BIG_ENDIAN)
			default_value = ((default_value & 0xff000000) >> 24) | ((default_value & 0x00ff0000) >> 8 ) | ((default_value & 0x0000ff00) << 8 ) | ((default_value & 0x000000ff) << 24);
	}
	else if(LastError == ERROR_SUCCESS)
		LastError = ERROR_INVALID_DATA;
	free(reg_value.Data);
	return default_value;
}


//////////////////////////////////////////////////////////////////////

QWORD  cls_Registry::GetQWord(LPCTSTR lpValueName, QWORD default_value)
{
	RegValue  reg_value;
	if(GetValue(lpValueName, reg_value) && reg_value.Type == REG_QWORD)
		default_value = *(QWORD *)reg_value.Data;
	else if(LastError == ERROR_SUCCESS)
		LastError = ERROR_INVALID_DATA;
	free(reg_value.Data);
	return default_value;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::SetBinary(LPCTSTR lpValueName, DWORD type, LPBYTE value, DWORD length)
{
	return SetValue(lpValueName, RegValue(type, value, length));
}

//////////////////////////////////////////////////////////////////////

bool  cls_Registry::SetString(LPCTSTR lpValueName, DWORD type, LPCTSTR value)
{
	DWORD  length;
	if(type == REG_MULTI_SZ)
	{
		LPCTSTR  p = value + 1;
		while(p[-1] && p[0])
			++p;
#pragma warning(disable:4244)
		length = p - value + 1;
#pragma warning(default:4244)
	}
	else
		length = lstrlen(value) + 1;
	return SetValue(lpValueName, RegValue(type, LPBYTE(value), length * sizeof(TCHAR)));
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::SetDWord(LPCTSTR lpValueName, DWORD type, DWORD value)
{
	if(type == REG_DWORD_BIG_ENDIAN)
		value = ((value & 0xff000000) >> 24) | ((value & 0x00ff0000) >> 8 ) | ((value & 0x0000ff00) << 8 ) | ((value & 0x000000ff) << 24);
	return SetValue(lpValueName, RegValue(type, LPBYTE(&value), sizeof(DWORD)));
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::SetQWord(LPCTSTR lpValueName, DWORD type, QWORD value)
{
	return SetValue(lpValueName, RegValue(type, LPBYTE(&value), sizeof(QWORD)));
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

LPTSTR  cls_Registry::GetRegSubKey(HKEY hKey, DWORD index)
{
	LPTSTR  buf = NULL;
	DWORD   ret, buf_size;
	ret = (DWORD)RegQueryInfoKey(hKey, NULL, NULL, NULL, NULL, &buf_size, NULL, NULL, NULL, NULL, NULL, NULL);
	if(ret != ERROR_SUCCESS)
	{
		SetLastError(ret);
		return NULL;
	}

	buf = (LPTSTR)malloc(++buf_size * sizeof(TCHAR));
	if(!buf)
	{
		SetLastError(ERROR_NOT_ENOUGH_MEMORY);
		return NULL;
	}
	FILETIME  dummy;
	DWORD     size = buf_size;
	ret = (DWORD)RegEnumKeyEx(hKey, index, buf, &size, NULL, NULL, NULL, &dummy);
	if(ret == ERROR_SUCCESS)
	{
		if(buf_size != ++size)
		{
			LPTSTR  temp = (LPTSTR)realloc(buf, size * sizeof(TCHAR));
			if(!temp)
			{
				ret = ERROR_NOT_ENOUGH_MEMORY;
				goto Fin;
			}
			buf = temp;
		}
		SetLastError(ERROR_SUCCESS);
		return buf;
	}
Fin:
	free(buf);
	SetLastError(ret);
	return NULL;
}


//////////////////////////////////////////////////////////////////////

LPTSTR  cls_Registry::GetRegValueName(HKEY hKey, DWORD index)
{
	bool    retvalue = false;
	LPTSTR  buf = NULL;
	DWORD   ret, size, buf_size;
	ret = (DWORD)RegQueryInfoKey(hKey, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &buf_size, NULL, NULL, NULL);
	if(ret != ERROR_SUCCESS)
		goto Fin;

	buf = (LPTSTR)malloc(++buf_size * sizeof(TCHAR));
	if(!buf)
	{
		ret = ERROR_NOT_ENOUGH_MEMORY;
		goto Fin;
	}
	size = buf_size;
	ret = (DWORD)RegEnumValue(hKey, index, buf, &size, NULL, NULL, NULL, NULL);
	if(ret != ERROR_SUCCESS)
		goto Fin;
	if(++size != buf_size)
	{
		LPTSTR  temp = (LPTSTR)realloc(buf, size * sizeof(TCHAR));
		if(!temp)
		{
			ret = ERROR_NOT_ENOUGH_MEMORY;
			goto Fin;
		}
		buf = temp;
	}
	retvalue = true;

Fin:
	if(!retvalue && buf)
	{
		free(buf);
		buf = NULL;
	}
	SetLastError(ret);
	return buf;
}



//////////////////////////////////////////////////////////////////////

bool  cls_Registry::GetRegValue(HKEY hKey, DWORD index, RegValue &reg_value)
{
	LPTSTR  buf = GetRegValueName(hKey, index);
	if(buf)
	{
		GetRegValue(hKey, buf, reg_value);
		DWORD  ret = GetLastError();
		free(buf);
		return ret == ERROR_SUCCESS;
	}
	return false;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::GetRegValue(HKEY hKey, LPCTSTR lpValueName, RegValue &reg_value)
{
	bool    retvalue = false;
	LPBYTE  buf = NULL;
	DWORD   type, size, ret, buf_size;
	ret = (DWORD)RegQueryInfoKey(hKey, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &buf_size, NULL, NULL);
	if(ret != ERROR_SUCCESS)
		goto Fin;

	buf = (LPBYTE)malloc(buf_size);
	if(!buf)
	{
		ret = ERROR_NOT_ENOUGH_MEMORY;
		goto Fin;
	}
	size = ++buf_size;
	ret = (DWORD)RegQueryValueEx(hKey, lpValueName, NULL, &type, buf, &size);
	if(ret != ERROR_SUCCESS)
		goto Fin;
	if(size != buf_size)
	{
		LPBYTE  temp = (LPBYTE)realloc(buf, size);
		if(!temp)
		{
			ret = ERROR_NOT_ENOUGH_MEMORY;
			goto Fin;
		}
		buf = temp;
	}

	retvalue = true;

Fin:
	if(!retvalue)
		free(buf);
	else
	{
		reg_value.Type = type;
		reg_value.Data = buf;
		reg_value.Size = size;
	}
	SetLastError(ret);
	return retvalue;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::SetRegValue(HKEY hKey, DWORD index, const RegValue &reg_value)
{
	LPTSTR  buf = GetRegValueName(hKey, index);
	if(buf)
	{
		SetRegValue(hKey, buf, reg_value);
		DWORD  ret = GetLastError();
		free(buf);
		return ret == ERROR_SUCCESS;
	}
	return false;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::SetRegValue(HKEY hKey, LPCTSTR lpValueName, const RegValue &reg_value)
{
	DWORD  ret = (DWORD)RegSetValueEx(hKey, lpValueName, NULL, reg_value.Type, reg_value.Data, reg_value.Size);
	SetLastError(ret);
	return ret == ERROR_SUCCESS;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::DeleteRegValue(HKEY hKey, LPCTSTR lpValueName)
{
	DWORD  ret = (DWORD)RegDeleteValue(hKey, lpValueName);
	SetLastError(ret);
	return ret == ERROR_SUCCESS;
}


//////////////////////////////////////////////////////////////////////

HKEY  cls_Registry::CreateRegKey(HKEY hKey, LPCTSTR lpKeyName)
{
	HKEY   hKey_ret;
	DWORD  ret = (DWORD)RegCreateKeyEx(hKey, lpKeyName, NULL, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey_ret, NULL);
	SetLastError(ret);
	if(ret == ERROR_SUCCESS)
		return hKey_ret;
	return NULL;
}


//////////////////////////////////////////////////////////////////////

HKEY  cls_Registry::OpenRegKey(HKEY hKey, LPCTSTR lpKeyName)
{
	HKEY   hKey_ret;
	DWORD  ret = (DWORD)RegOpenKeyEx(hKey, lpKeyName, 0, KEY_ALL_ACCESS, &hKey_ret);
	SetLastError(ret);
	if(ret == ERROR_SUCCESS)
		return hKey_ret;
	return NULL;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::DeleteRegKey(HKEY hKey, LPCTSTR lpKeyName)
{
	DWORD  ret = (DWORD)RegDeleteKey(hKey, lpKeyName);
	SetLastError(ret);
	return ret == ERROR_SUCCESS;
}


//////////////////////////////////////////////////////////////////////

bool  cls_Registry::DeleteRegKeyTree(HKEY hKey, LPCTSTR lpKeyName)
{
	HKEY  hKey_sub = OpenRegKey(hKey, lpKeyName);
	if(!hKey_sub)
		return false;
	DWORD  ret = ERROR_SUCCESS;
	for(; ; )
	{
		LPTSTR  buf = GetRegSubKey(hKey_sub, 0);
		ret = GetLastError();
		if(!buf)
			break;
		DeleteRegKeyTree(hKey_sub, buf);
		ret = GetLastError();
		free(buf);
		if(ret != ERROR_SUCCESS)
			break;
	}
	RegCloseKey(hKey_sub);
	if(ret == ERROR_SUCCESS || ret == ERROR_NO_MORE_ITEMS)
		ret = (DWORD)RegDeleteKey(hKey, lpKeyName);
	SetLastError(ret);
	return ret == ERROR_SUCCESS;
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
