//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#ifndef  INC_REGISTOY
#define  INC_REGISTRY


//////////////////////////////////////////////////////////////////////

#include  <windows.h>


//////////////////////////////////////////////////////////////////////

#define  REG_ERROR  -1

#ifndef  REG_QWORD
#define  REG_QWORD  11
#endif

#ifndef  REG_QWORD_LITTLE_ENDIAN
#define  REG_QWORD_LITTLE_ENDIAN  11
#endif

#ifndef  QWORD
#define  QWORD  ULONG64
#endif


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

struct RegValue
{
	DWORD   Type;
	LPBYTE  Data;
	DWORD   Size;
	RegValue(                                   ): Type(REG_ERROR), Data(NULL), Size(0   ) { }
	RegValue(DWORD type, LPBYTE data, DWORD size): Type(type     ), Data(data), Size(size) { }
};


//////////////////////////////////////////////////////////////////////

class  cls_Registry
{
	public:
		cls_Registry(): hKey(NULL), NoClose(true), LastError(ERROR_SUCCESS) { }
		cls_Registry(HKEY new_hKey, LPCTSTR new_key, bool create_sw = false, bool no_close = false);
		~cls_Registry() { Close(); }

	private:
		cls_Registry(const cls_Registry &);
		cls_Registry &operator =(const cls_Registry &);

	public:
		bool    Open(HKEY new_hKey, LPCTSTR new_key, bool no_close = false);
		bool    Create(HKEY new_hKey, LPCTSTR new_key);
		void    Close();
		DWORD   SubKeyCount();
		DWORD   ValueCount();
		LPTSTR  GetSubKey(DWORD index) { LPTSTR  ret = GetRegSubKey(hKey, index); LastError = GetLastError(); return ret; }
		LPTSTR  GetValueName(DWORD index) { LPTSTR  ret = GetRegValueName(hKey, index); LastError = GetLastError(); return ret; }
		bool    GetValue(LPCTSTR lpValueName, RegValue &reg_value) { bool  ret = GetRegValue(hKey, lpValueName, reg_value); LastError = GetLastError(); return ret; }
		HKEY    MakeSubKey(LPCTSTR lpKeyName) { HKEY  ret = CreateRegKey(hKey, lpKeyName); LastError = GetLastError(); return ret; }
		bool    SetValue(LPCTSTR lpValueName, const RegValue &reg_value) { bool  ret = SetRegValue(hKey, lpValueName, reg_value); LastError = GetLastError(); return ret; }
		bool    DeleteSubKey(LPCTSTR lpKeyName) { bool  ret = DeleteRegKey(hKey, lpKeyName); LastError = GetLastError(); return ret; }
		bool    DeleteSubKeyTree(LPCTSTR lpKeyName) { bool  ret = DeleteRegKeyTree(hKey, lpKeyName); LastError = GetLastError(); return ret; }
		bool    DeleteValue(LPCTSTR lpValueName) { bool  ret = DeleteRegValue(hKey, lpValueName); LastError = GetLastError(); return ret; }
		bool    DeleteAllSubKeys();
		bool    DeleteAllValues();

	public:
		bool  operator !() { return hKey == NULL || LastError != ERROR_SUCCESS; }
		operator bool() { return !operator !(); }

	public:
		LPBYTE  GetBinary(LPCTSTR lpValueName, DWORD &length);
		LPTSTR  GetString(LPCTSTR lpValueName);
		DWORD   GetDWord(LPCTSTR lpValueName, DWORD default_value = 0);
		QWORD   GetQWord(LPCTSTR lpValueName, QWORD default_value = 0);
		bool    SetBinary(LPCTSTR lpValueName, DWORD type, LPBYTE value, DWORD length);
		bool    SetString(LPCTSTR lpValueName, DWORD type, LPCTSTR value);
		bool    SetDWord(LPCTSTR lpValueName, DWORD type, DWORD value);
		bool    SetQWord(LPCTSTR lpValueName, DWORD type, QWORD value);

		bool  SetNONE(LPCTSTR lpValueName, LPBYTE value, DWORD length) { return SetBinary(lpValueName, REG_NONE, value, length); }
		bool  SetSZ(LPCTSTR lpValueName, LPCTSTR value) { return SetString(lpValueName, REG_SZ, value); }
		bool  SetEXPAND_SZ(LPCTSTR lpValueName, LPCTSTR value) { return SetString(lpValueName, REG_EXPAND_SZ, value); }
		bool  SetBINARY(LPCTSTR lpValueName, LPBYTE value, DWORD length) { return SetBinary(lpValueName, REG_BINARY, value, length); }
		bool  SetDWORD(LPCTSTR lpValueName, DWORD value) { return SetDWord(lpValueName, REG_DWORD, value); }
		bool  SetDWORD_BIG_ENDIAN(LPCTSTR lpValueName, DWORD value) { return SetDWord(lpValueName, REG_DWORD_BIG_ENDIAN, value); }
		bool  SetLINK(LPCTSTR lpValueName, LPBYTE value, DWORD length) { return SetBinary(lpValueName, REG_LINK, value, length); }
		bool  SetMULTI_SZ(LPCTSTR lpValueName, LPCTSTR value) { return SetString(lpValueName, REG_MULTI_SZ, value); }
		bool  SetRESOURCE_LIST(LPCTSTR lpValueName, LPBYTE value, DWORD length) { return SetBinary(lpValueName, REG_RESOURCE_LIST, value, length); }
		bool  SetFULL_RESOURCE_DESCRIPTOR(LPCTSTR lpValueName, LPBYTE value, DWORD length) { return SetBinary(lpValueName, REG_FULL_RESOURCE_DESCRIPTOR, value, length); }
		bool  SetRESOURCE_REQUIREMENTS_LIST(LPCTSTR lpValueName, LPBYTE value, DWORD length) { return SetBinary(lpValueName, REG_RESOURCE_REQUIREMENTS_LIST, value, length); }
		bool  SetQWORD(LPCTSTR lpValueName, QWORD value) { return SetQWord(lpValueName, REG_QWORD, value); }

	public:
		HKEY   hKey;
		bool   NoClose;
		DWORD  LastError;

	public:
		static LPTSTR  GetRegSubKey(HKEY hKey, DWORD index);
		static LPTSTR  GetRegValueName(HKEY hKey, DWORD index);
		static bool    GetRegValue(HKEY hKey, DWORD index, RegValue &reg_value);
		static bool    GetRegValue(HKEY hKey, LPCTSTR lpValueName, RegValue &reg_value);
		static bool    SetRegValue(HKEY hKey, DWORD index, const RegValue &reg_value);
		static bool    SetRegValue(HKEY hKey, LPCTSTR lpValueName, const RegValue &reg_value);
		static bool    DeleteRegValue(HKEY hKey, LPCTSTR lpValueName);
		static HKEY    CreateRegKey(HKEY hKey, LPCTSTR lpKeyName);
		static HKEY    OpenRegKey(HKEY hKey, LPCTSTR lpKeyName);
		static bool    DeleteRegKey(HKEY hKey, LPCTSTR lpKeyName);
		static bool    DeleteRegKeyTree(HKEY hKey, LPCTSTR lpKeyName);
};


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#endif// INC_REGISTRY


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
